--------------------------------------------------------------------------------
-- Company: FSU-FAMU
-- Engineer: Dr. Uwe Meyer-Baese
-- Create Date:    10:56:41 01/01/11
-- Design Name:    i2c_controller
-- Project Name:    DE2_lab
-- Target Device:   EP2C35F672C6
-- Tool versions:   Quartus II 9.1
-- Description:   This is a parallel to serial converter for the I2C 
--                audio CODEC on the DE2 board. The 7 bit address (34h)
--                is followed by 16 bit data. After each bytes a acknoledge
--                signal (low) is returned from the CODEC
--
-- Dependencies:  The module is called by ic2_controller
--
-- Resources used:
--  Total Number LEs: 65         
--  Number of 9x9 Mults: 0
--  Number of Embedded 4Ks: 0
--  Maximum Delay is 3.61 ns => 276.32 MHz
--------------------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE ieee.std_logic_unsigned.ALL;
--------------------------------------------------------------------------------
ENTITY i2c_controller IS PORT(
    clock      : IN STD_LOGIC; -- Input clock from DE2 = 50 MHz
    i2c_sclk   : OUT STD_LOGIC; -- I2C clock
    i2c_sdat   : INOUT STD_LOGIC; -- I2C data
    i2c_data   : IN STD_LOGIC_VECTOR(23 DOWNTO 0); -- DATA:[SLAVE_ADDR,SUB_ADDR,DATA]
    go         : IN STD_LOGIC;     -- Transfer go
    done       : OUT STD_LOGIC;    -- Transfer done
    ack        : OUT STD_LOGIC;    -- Acknowldge from CODEC
    reset      : IN STD_LOGIC;
    sd_counter : BUFFER STD_LOGIC_VECTOR(5 DOWNTO 0);
    sdo        : BUFFER STD_LOGIC);
END ENTITY i2c_controller;

ARCHITECTURE fpga OF i2c_controller IS

  SIGNAL SD : STD_LOGIC_VECTOR(23 DOWNTO 0); -- Register for parallel in data
  SIGNAL ACK1, ACK2, ACK3 : STD_LOGIC;  -- Single acknoldge after each byte
  SIGNAL SCLK : STD_LOGIC; -- Clock signal
  
BEGIN 
----------------------  Clock and output enable ----------------------------
 P1: PROCESS(sclk, sd_counter, clock, sdo)
 BEGIN
   IF sclk='1' OR (sd_counter >= 4 and sd_counter <=30) THEN
     i2c_sclk <= NOT clock; 
   ELSE 
     i2c_sclk <=  '0';
   END IF;
   IF sdo = '1' THEN
     i2c_sdat <= 'Z';
   ELSE
     i2c_sdat <= '0';
   END IF;
 END PROCESS; 
 ack <= ack1 or ack2 or ack3;
 
---------------------- I2C counter ----------------------
P2: PROCESS
BEGIN
  WAIT UNTIL clock = '1';
  IF reset = '0' THEN 
    sd_counter <= (OTHERS => '1');
  ELSIF GO = '0' THEN 
    sd_counter <= (OTHERS => '0');
    ELSIF sd_counter < "0111111" THEN
      sd_counter <= sd_counter + 1;    
  END IF;
END PROCESS;

---------------------- Parallel -> serial conversion ----------------------
P3: PROCESS
BEGIN
  WAIT UNTIL CLOCK = '1';
  IF reset = '0' THEN 
    sclk <= '1'; sdo <= '1'; ack1 <= '0'; 
    ack2 <= '0'; ack3 <= '0'; done <= '1';
  ELSE
    CASE CONV_INTEGER(sd_counter) IS
    WHEN  0  => ack1 <= '0'; ack2 <= '0'; ack3 <= '0'; 
                done <= '0'; sdo <= '1'; sclk <= '1';
    -- Start tranfer
    WHEN  1  => sdo <= '0'; sd <= i2c_data; -- Take over input 24 bit vector
    WHEN  2  => sclk <= '0';
    -- I2C component address
    WHEN  3  => sdo <= SD(23);
    WHEN  4  => sdo <= sd(22);
    WHEN  5  => sdo <= sd(21);
    WHEN  6  => sdo <= sd(20);
    WHEN  7  => sdo <= sd(19);
    WHEN  8  => sdo <= sd(18);
    WHEN  9  => sdo <= sd(17);
    WHEN  10 => sdo <= sd(16); 
    WHEN  11 => sdo <= 'Z'; -- ack1 precharge; read input next edge
    -- MSBs data
    WHEN  12  => sdo <= sd(15); ACK1 <= I2C_SDAT;
    WHEN  13  => sdo <= sd(14);
    WHEN  14  => sdo <= sd(13);
    WHEN  15  => sdo <= sd(12);
    WHEN  16  => sdo <= sd(11);
    WHEN  17  => sdo <= sd(10);
    WHEN  18  => sdo <= sd(9);
    WHEN  19  => sdo <= sd(8);
    WHEN  20  => sdo <= 'Z'; -- ack2 precharge; read input next edge
    -- LSBs data
    WHEN  21  => sdo <= sd(7); ack2 <= I2C_SDAT; 
    WHEN  22  => sdo <= sd(6);
    WHEN  23  => sdo <= sd(5);
    WHEN  24  => sdo <= sd(4);
    WHEN  25  => sdo <= sd(3);
    WHEN  26  => sdo <= sd(2);
    WHEN  27  => sdo <= sd(1);
    WHEN  28  => sdo <= sd(0);
    WHEN  29  => sdo <= 'Z'; -- ACK3 precharge; read input next edge  
    -- Tail sequence
    WHEN  30 => sdo <= '0'; sclk <= '0'; ack3 <= i2c_sdat;
    WHEN  31 => sclk <= '1'; 
    WHEN  OTHERS => sdo <= '1'; done <= '1'; 
    END CASE;
  END IF;
END PROCESS;

END ARCHITECTURE fpga;
