--------------------------------------------------------------------------------
-- Company: FSU-FAMU
-- Engineer: Dr. Uwe Meyer-Baese
-- Create Date:    10:56:41 01/01/11
-- Design Name:    i2c_config
-- Project Name:    DE2_lab
-- Target Device:   EP2C35F672C6
-- Tool versions:   Quartus II 9.1
-- Description:   This is the top level of the I2C driver.
--                It take the program words and calls the
--                i2c_controller for the serial transfer.
--
-- Dependencies:  The module calls the ic2_controller
--
-- Resources used:
--  Total Number LEs: 100         
--  Number of 9x9 Mults: 0
--  Number of Embedded 4Ks: 0
--  Maximum Delay is 3.288 ns => 304.14 MHz
--------------------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE ieee.std_logic_unsigned.ALL;
--------------------------------------------------------------------------------
ENTITY i2c_config IS  
  PORT (iclk     : IN STD_LOGIC; --  Host Side
        irst_n   : IN STD_LOGIC;
        i2c_sclk : OUT STD_LOGIC; --  I2C Side
        i2c_sdat : INOUT STD_LOGIC);
END ENTITY I2C_Config;
--------------------------------------------------------------------------------
ARCHITECTURE fpga OF I2C_Config IS

COMPONENT i2c_controller IS
  PORT(
    clock      : IN STD_LOGIC; -- Input clock from DE2 = 50 MHz
    i2c_sclk   : OUT STD_LOGIC; -- I2C clock
    i2c_sdat   : INOUT STD_LOGIC; -- I2C data
    i2c_data   : IN STD_LOGIC_VECTOR(23 DOWNTO 0); -- DATA:[SLAVE_ADDR,SUB_ADDR,DATA]
    go         : IN STD_LOGIC;       -- Transfer go
    done       : OUT STD_LOGIC;    -- Transfer done
    ack        : OUT STD_LOGIC;     -- Acknoldge 
    reset      : IN STD_LOGIC;
    sd_counter : BUFFER STD_LOGIC_VECTOR(5 DOWNTO 0);
    sdo        : BUFFER STD_LOGIC);
END COMPONENT;

--  Internal Registers/Wires
  SIGNAL mi2c_clk_div, lut_data : STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL mi2c_data : STD_LOGIC_VECTOR(23 DOWNTO 0);
  SIGNAL mi2c_ctrl_clk, mi2c_go, mi2c_end, mi2c_ack : STD_LOGIC;
  SIGNAL lut_index : STD_LOGIC_VECTOR(5 DOWNTO 0);
  SIGNAL msetup_st : STD_LOGIC_VECTOR(1 DOWNTO 0);  
--  Clock Setting
  CONSTANT  CLK_FREQ     : INTEGER := 50000000;   --  50  MHz
  CONSTANT  I2C_FREQ     : INTEGER :=    20000;   --  20  KHz
--CONSTANT  I2C_FREQ     : INTEGER :=  5000000;   --  5  MHz for simulation only
--  LUT Data Number
  CONSTANT  LUT_SIZE     : INTEGER :=   11;
--  Audio Data Index
  CONSTANT  Dummy_DATA   : INTEGER :=   0;
  CONSTANT  SET_LIN_L    : INTEGER :=   1;
  CONSTANT  SET_LIN_R    : INTEGER :=   2;
  CONSTANT  SET_HEAD_L   : INTEGER :=   3;
  CONSTANT  SET_HEAD_R   : INTEGER :=   4;
  CONSTANT  A_PATH_CTRL  : INTEGER :=   5;
  CONSTANT  D_PATH_CTRL  : INTEGER :=   6;
  CONSTANT  POWER_ON     : INTEGER :=   7;
  CONSTANT  SET_FORMAT   : INTEGER :=   8;
  CONSTANT  SAMPLE_CTRL  : INTEGER :=   9;
  CONSTANT  SET_ACTIVE   : INTEGER :=   10;
  
BEGIN
--------------------   I2C Control Clock   ------------------------
  P1: PROCESS
  BEGIN
    WAIT UNTIL iclk = '1';
    IF irst_n = '0' THEN 
      mi2c_ctrl_clk   <= '0';
      mi2c_clk_div    <= (OTHERS => '0');
    ELSIF mi2c_clk_div < CLK_FREQ/I2C_FREQ THEN
          mi2c_clk_div <= mi2c_clk_div + 1;
    ELSE
      mi2c_clk_div    <=  (OTHERS => '0');
      mi2c_ctrl_clk   <=  NOT mi2c_ctrl_clk;
    END IF;
  END PROCESS;
--------------------------------------------------------------------
C1: I2C_Controller PORT MAP(
                clock => mi2c_ctrl_clk,   --  Controller work clock
                i2c_sclk => i2c_sclk,     --  I2C clock
                i2c_sdat => i2c_sdat,     --  I2C data
                i2c_data => mi2c_data,    --  DATA:[SLAVE_ADDR,SUB_ADDR,DATA]
                go => mi2c_go,            --  transfer go
                done => mi2c_end,         --  transfer end 
                ack => mi2c_ack,          --  Acknowldge from CODEC
                reset => irst_n);
--------------------------------------------------------------------
----------------------  Config Control  ----------------------------
  P2: PROCESS
  BEGIN
    WAIT UNTIL mI2C_CTRL_CLK = '1';
    IF iRST_N = '0' THEN 
      lut_index <= (OTHERS => '0');
      msetup_st <= (OTHERS => '0');
      mi2c_go   <= '0';
    ELSIF LUT_INDEX < LUT_SIZE THEN
      CASE CONV_INTEGER(mSetup_ST) IS
      WHEN 0 => mi2c_data   <=  X"34" & LUT_DATA; -- 34 hex is is the audio codec
                mi2c_go     <=  '1';
                msetup_st   <=  "01";
      WHEN 1 => IF mi2c_end = '1' THEN -- Do all bits
                  mi2c_go     <=  '0';
                  IF mi2c_ack = '0' THEN -- Without acknowldge repeat procedure
                    msetup_st <= "10";
                  ELSE
                    msetup_st <= (OTHERS => '0');                          
                  END IF;
                END IF;
      WHEN OTHERS => lut_index <= lut_index + 1; -- Program next register 
                     msetup_st <= (OTHERS => '0');
      END CASE;
    END IF;
  END PROCESS;
--------------------------------------------------------------------
---------------------   Config Data LUT   --------------------------    
  P3: PROCESS(LUT_INDEX)
  BEGIN
    CASE CONV_INTEGER(LUT_INDEX) IS --  Audio config data
    WHEN DUMMY_DATA  => LUT_DATA <= X"0000";
    WHEN SET_LIN_L   => LUT_DATA <= X"001A";
    WHEN SET_LIN_R   => LUT_DATA <= X"021A";
    WHEN SET_HEAD_L  => LUT_DATA <= X"047B";
    WHEN SET_HEAD_R  => LUT_DATA <= X"067B";
    WHEN A_PATH_CTRL => LUT_DATA <= X"08F8";
    WHEN D_PATH_CTRL => LUT_DATA <= X"0A06";
    WHEN POWER_ON    => LUT_DATA <= X"0C00";
    WHEN SET_FORMAT  => LUT_DATA <= X"0E01";
    WHEN SAMPLE_CTRL => LUT_DATA <= X"1002";
    WHEN SET_ACTIVE  => LUT_DATA <= X"1201";
    WHEN OTHERS      => LUT_DATA <= X"0000";
    END CASE;
  END PROCESS;
  
END ARCHITECTURE fpga;
